# MyTT 麦语言-通达信-同花顺指标实现     https://github.com/mpquant/MyTT
# 高级函数版本，本文件函数计算结果经过验证完全正确，可以正常使用，但代码比较复杂，做为进阶使用。
# MyTT团队对每个函数精益求精，力争效率速度，代码优雅的完美统一，如果您有更好的实现方案，请不吝赐教！
# 感谢以下团队成员的努力和贡献： 火焰，jqz1226, stanene, bcq
import math

import numpy as np
import pandas as pd
from deprecated.sphinx import deprecated

from quant1x.base.formula import SUM, DMA, REF


# ------------------------工具函数---------------------------------------------

@deprecated(version="0.1.0", reason="合并到base里面, 以后不会单独出现序列版本")
def HHV(S, N):  # HHV,支持N为序列版本
    # type: (np.ndarray, Optional[int,float, np.ndarray]) -> np.ndarray
    """
    HHV(C, 5)  # 最近5天收盘最高价
    """
    if isinstance(N, (int, float)):
        return pd.Series(S).rolling(N).max().values
    else:
        res = np.repeat(np.nan, len(S))
        for i in range(len(S)):
            if (not np.isnan(N[i])) and N[i] <= i + 1:
                res[i] = S[i + 1 - N[i]:i + 1].max()
        return res

@deprecated(version="0.1.0", reason="合并到base里面, 以后不会单独出现序列版本")
def LLV(S, N):  # LLV,支持N为序列版本
    # type: (np.ndarray, Optional[int,float, np.ndarray]) -> np.ndarray
    """
    LLV(C, 5)  # 最近5天收盘最低价
    """
    if isinstance(N, (int, float)):
        return pd.Series(S).rolling(N).min().values
    else:
        res = np.repeat(np.nan, len(S))
        for i in range(len(S)):
            if (not np.isnan(N[i])) and N[i] <= i + 1:
                res[i] = S[i + 1 - N[i]:i + 1].min()
        return res


def DSMA(X, N):  # 偏差自适应移动平均线   type: (np.ndarray, int) -> np.ndarray
    """
    Deviation Scaled Moving Average (DSMA)    Python by: jqz1226, 2021-12-27
    Referred function from myTT: SUM, DMA
    """
    a1 = math.exp(- 1.414 * math.pi * 2 / N)
    b1 = 2 * a1 * math.cos(1.414 * math.pi * 2 / N)
    c2 = b1
    c3 = -a1 * a1
    c1 = 1 - c2 - c3
    Zeros = np.pad(X[2:] - X[:-2], (2, 0), 'constant')
    Filt = np.zeros(len(X))
    for i in range(len(X)):
        Filt[i] = c1 * (Zeros[i] + Zeros[i - 1]) / 2 + c2 * Filt[i - 1] + c3 * Filt[i - 2]

    RMS = np.sqrt(SUM(np.square(Filt), N) / N)
    ScaledFilt = Filt / RMS
    alpha1 = np.abs(ScaledFilt) * 5 / N
    return DMA(X, alpha1)


def SUMBARSFAST(X, A):
    # type: (np.ndarray, Optional[np.ndarray, float, int]) -> np.ndarray
    """
    通达信SumBars函数的Python实现  by jqz1226
    SumBars函数将X向前累加，直到大于等于A, 返回这个区间的周期数。例如SUMBARS(VOL, CAPITAL),求完全换手的周期数。
    :param X: 数组。被累计的源数据。 源数组中不能有小于0的元素。
    :param A: 数组（一组）或者浮点数（一个）或者整数（一个），累加截止的界限数
    :return:  数组。各K线分别对应的周期数
    """
    if any(X <= 0):   raise ValueError('数组X的每个元素都必须大于0！')

    X = np.flipud(X)  # 倒转
    length = len(X)

    if isinstance(A * 1.0, float):  A = np.repeat(A, length)  # 是单值则转化为数组
    A = np.flipud(A)  # 倒转
    sumbars = np.zeros(length)  # 初始化sumbars为0
    Sigma = np.insert(np.cumsum(X), 0, 0.0)  # 在累加值前面插入一个0.0（元素变多1个，便于引用）

    for i in range(length):
        k = np.searchsorted(Sigma[i + 1:], A[i] + Sigma[i])
        if k < length - i:  # 找到
            sumbars[length - i - 1] = k + 1
    return sumbars.astype(int)


# ------------------------指标函数---------------------------------------------

def SAR(HIGH, LOW, N=10, S=2, M=20):
    """
    求抛物转向。 例如SAR(10,2,20)表示计算10日抛物转向，步长为2%，步长极限为20%
    Created by: jqz1226, 2021-11-24首次发表于聚宽(www.joinquant.com)
    
    :param HIGH: high序列
    :param LOW: low序列
    :param N: 计算周期
    :param S: 步长
    :param M: 步长极限
    :return: 抛物转向
    """
    f_step = S / 100
    f_max = M / 100
    af = 0.0
    is_long = HIGH[N - 1] > HIGH[N - 2]
    b_first = True
    length = len(HIGH)

    s_hhv = REF(HHV(HIGH, N), 1)  # type: np.ndarray
    s_llv = REF(LLV(LOW, N), 1)  # type: np.ndarray
    sar_x = np.repeat(np.nan, length)  # type: np.ndarray
    for i in range(N, length):
        if b_first:  # 第一步
            af = f_step
            sar_x[i] = s_llv[i] if is_long else s_hhv[i]
            b_first = False
        else:  # 继续多 或者 空
            ep = s_hhv[i] if is_long else s_llv[i]  # 极值
            if (is_long and HIGH[i] > ep) or ((not is_long) and LOW[i] < ep):  # 顺势：多创新高 或者 空创新低
                af = min(af + f_step, f_max)
            #
            sar_x[i] = sar_x[i - 1] + af * (ep - sar_x[i - 1])

        if (is_long and LOW[i] < sar_x[i]) or ((not is_long) and HIGH[i] > sar_x[i]):  # 反空 或者 反多
            is_long = not is_long
            b_first = True
    return sar_x


def TDX_SAR(High, Low, iAFStep=2, iAFLimit=20):  # type: (np.ndarray, np.ndarray, int, int) -> np.ndarray
    """  通达信SAR算法,和通达信SAR对比完全一致   by: jqz1226, 2021-12-18
    :param High: 最高价序列
    :param Low: 最低价序列
    :param iAFStep: AF步长
    :param iAFLimit: AF极限值
    :return: SAR序列
    """
    af_step = iAFStep / 100
    af_limit = iAFLimit / 100
    SarX = np.zeros(len(High))  # 初始化返回数组

    # 第一个bar
    bull = True
    af = af_step
    ep = High[0]
    SarX[0] = Low[0]
    # 第2个bar及其以后
    for i in range(1, len(High)):
        # 1.更新：hv, lv, af, ep
        if bull:  # 多
            if High[i] > ep:  # 创新高
                ep = High[i]
                af = min(af + af_step, af_limit)
        else:  # 空
            if Low[i] < ep:  # 创新低
                ep = Low[i]
                af = min(af + af_step, af_limit)
        # 2.计算SarX
        SarX[i] = SarX[i - 1] + af * (ep - SarX[i - 1])

        # 3.修正SarX
        if bull:
            SarX[i] = max(SarX[i - 1], min(SarX[i], Low[i], Low[i - 1]))
        else:
            SarX[i] = min(SarX[i - 1], max(SarX[i], High[i], High[i - 1]))

        # 4. 判断是否：向下跌破，向上突破
        if bull:  # 多
            if Low[i] < SarX[i]:  # 向下跌破，转空
                bull = False
                tmp_SarX = ep  # 上阶段的最高点
                ep = Low[i]
                af = af_step
                if High[i - 1] == tmp_SarX:  # 紧邻即最高点
                    SarX[i] = tmp_SarX
                else:
                    SarX[i] = tmp_SarX + af * (ep - tmp_SarX)
        else:  # 空
            if High[i] > SarX[i]:  # 向上突破, 转多
                bull = True
                ep = High[i]
                af = af_step
                SarX[i] = min(Low[i], Low[i - 1])
    # end for
    return SarX



#!/usr/bin/python
# -*- coding: UTF-8 -*-

quant1x_home = '~/.quant1x'
quant1x_data = quant1x_home + '/data'
quant1x_data_cn = quant1x_data + '/cn'
quant1x_data_hk = quant1x_data + '/hk'
quant1x_info = quant1x_home + '/info'
quant1x_info_cn = quant1x_info + '/cn'
quant1x_info_hk = quant1x_info + '/hk'

from quant1x.data import quotes

# export
D = quotes.DataHandler()



#!/usr/bin/python
# -*- coding: UTF-8 -*-

"""
89K 努力打造一个适度的量化交易框架
"""

__version__ = "0.1.0"
__author__ = "WangFeng"

import sys

if sys.version_info < (3, 8):
    print(f"Quant1X {__version__} requires Python 3.8+ and 64 bit OS")
    sys.exit(1)
del sys


def init(**kwargs):
    # qx.data.loader.DataHandler()
    pass



#!/usr/bin/python
# -*- coding: UTF-8 -*-
import pandas

from quant1x.base.formula import *

np.seterr(divide='ignore', invalid='ignore')
def cdtd(data: pandas.DataFrame):
    """
    抄底逃顶
    :param data:
    :return:
    """
    df = data.copy()
    OPEN = df['open']
    CLOSE = df['close']
    HIGH = df['high']
    LOW = df['low']
    VOL = df['volume']
    AMOUNT = df['amount']

    # N1:=3;
    N1 = 3
    # N2:=9;
    N2 = 9
    # N3:=27;
    N3 = 27
    # N4:=5;
    N4 = 5
    # HHVLLV3:=HHV(HIGH,N3)-LLV(LOW,N3);
    HHVLLV3 = HHV(HIGH, N3) - LLV(LOW, N3)
    #df['cdtd.h3'] = HHV(HIGH, N3)
    #df['cdtd.l3'] = LLV(LOW, N3)
    #df['cdtd.hl3'] = HHVLLV3
    # CLLV3:=CLOSE-LLV(LOW,N3);
    CLLV3 = CLOSE - LLV(LOW, N3)
    # RSV1:=(CLOSE-LLV(LOW,N2))/(HHV(HIGH,N2)-LLV(LOW,N2))*100;
    RSV1 = (CLOSE - LLV(LOW, N2)) / (HHV(HIGH, N2) - LLV(LOW, N2)) * 100
    # RSV2:=CLLV3/HHVLLV3*100;
    RSV2 = CLLV3 / HHVLLV3 * 100
    # WEN:=N1*SMA(RSV2,N4,1)-2*SMA(SMA(RSV2,N4,1),N1,1);
    WEN = N1 * SMA(RSV2, N4, 1) - 2 * SMA(SMA(RSV2, N4, 1), N1, 1)
    df['cdtd.wen']= WEN
    # J1:=SMA(RSV1,N1,1);
    J1 = SMA(RSV1, N1, 1)
    # J2:=SMA(J1,N1,1);
    J2 = SMA(J1, N1, 1)
    # W1:=SMA(RSV2,N1,1);
    W1 = SMA(RSV2, N1, 1)
    # W2:=SMA(W1,N1,1);
    W2 = SMA(W1, N1, 1)
    # 强弱界线:49,DOTLINE,LINETHICK1,COLOR9966CC;
    # 顶:100, DOTLINE,COLORCCFF00;
    # 底:0, DOTLINE, COLORRED;
    # 趋势线:WEN,LINETHICK1,COLORFF84FF;
    # 卖出:=CROSS(J2,J1) AND J2>85;
    S1 = CROSS(J2, J1)
    S2 = J2 > 85
    S = S1 & S2
    df['cdtd.S1'] = S
    # 卖点预警线:90,DOTLINE,LINETHICK1,COLORBLUE;
    # DRAWICON(卖出, 70, 2);
    # 买入:=趋势线<REF(趋势线,1) AND 趋势线<=5;
    B1 = WEN < REF(WEN, 1)
    B2 = WEN <= 5
    B = B1 & B2
    df['cdtd.B'] = B
    # 买点预警线:10,DOTLINE,LINETHICK1,COLORWHITE;
    # DRAWICON(买入, 30, 1);
    # G1:=W1{,LINETHICK2,COLORWHITE};
    # G2:=W2{,LINETHICK2,COLORCYAN};
    # STICKLINE(趋势线>=85,100,趋势线,5,1),COLORGREEN;
    # STICKLINE(趋势线<=5,0,趋势线,5,1),COLORYELLOW;
    df['cdtd.S2'] = B2
    # STICKLINE(COUNT(趋势线<REF(趋势线,1) AND 趋势线<=5,2)=2,0,20,8,0),COLORRED;
    df['cdtd.S3'] = COUNT(B1 & B2,2)==2
    # STICKLINE(CROSS(J2,J1) AND J2>=85,100,80,8,0),COLORGREEN;
    return df



#!/usr/bin/python
# -*- coding: UTF-8 -*-
import pandas
from quant1x.base import basic
from quant1x.base.formula import *

def hjk(code:str, data:pandas.DataFrame):
    """
    黄金坑
    """
    df = data
    OPEN = df['open']
    CLOSE = df['close']
    HIGH = df['high']
    LOW = df['low']
    VOL = df['volume']
    AMOUNT = df['amount']

    # ZTBL:="BASIC.涨停比例",NODRAW;
    ZTBL = basic.ZTBL(code)
    # R_CLOSE:=REF(CLOSE,1);
    R_CLOSE = REF(CLOSE, 1)
    # ZTJ:=ZTPRICE(R_CLOSE, ZTBL);
    ZTJ = basic.ZTPRICE(R_CLOSE, ZTBL)
    # BL:=VOL/REF(VOL,1),NODRAW;
    BL = VOL / REF(VOL, 1)
    # {倍量}
    # C11:=BL>=2.00,NODRAW;
    C11 = BL >= 2.00
    # {炸板}
    # C12:=CLOSE<=ZTJ,NODRAW;
    C12 = CLOSE <= ZTJ
    # {最高价大于涨停价的90%}
    # C13:=HIGH>=ZTJ*0.98,NODRAW;
    C13 = HIGH >= ZTJ * 0.98
    # PC:=C11 AND C12 AND C13,COLORYELLOW;
    #PC = pandas.eval('C11 & C12 & C13')
    PC = C11 & C12 & C13
    # PN:BARSLAST(PC),NODRAW;
    PN = BARSLAST(PC)
    # 前高:REF(CLOSE,PN),COLORWHITE;
    前高 = REF(CLOSE, PN)
    # 当前:CLOSE,COLORLIMAGENTA;
    当前 = CLOSE
    # 估值:FORCAST(CLOSE,PN),COLORYELLOW;
    估值 = FORCAST(CLOSE, PN)
    # 偏差:10*AVEDEV(CLOSE,PN),COLORLICYAN,NODRAW;
    偏差 = 10 * AVEDEV(CLOSE, PN)
    # PV:=REF(VOL,PN);
    PV = REF(VOL, PN)
    # PVH:=HHV(VOL,PN+1),NODRAW;
    PVH = HHV(VOL, PN + 1)
    # CJ:=COUNT(前高>=CLOSE,PN),NODRAW;
    CJ = COUNT(前高 >= CLOSE, PN)
    # SL:=VOL/PVH,NODRAW;
    SL = VOL / PVH
    # C21:SL<=0.50,NODRAW;
    C21 = SL <= 0.50
    # C22:CJ/PN>=0.50 OR CJ=PN,NODRAW;
    C22 = pandas.eval('CJ/PN>=0.50 | CJ==PN')
    # C23:PN>=4 AND PN<7,NODRAW;
    C23 = pandas.eval('PN>=4 & PN<7')
    # B:1*(C21 AND C22 AND C23),COLORRED,NODRAW;
    B = C21 & C22 & C23
    # DRAWICON(B>0,CLOSE*0.999,1);
    df['hjk.b'] = B
    # df['hjk.c21'] = C21
    # df['hjk.c22'] = C22
    # df['hjk.c23'] = C23
    return df


#!/usr/bin/python
# -*- coding: UTF-8 -*-

import pandas

from quant1x.formula import *


def ma1x(data:pandas.DataFrame):
    """
    89K趋势策略线
    :param data:
    :return:
    """
    df = data
    OPEN = df['open']
    CLOSE = df['close']
    HIGH = df['high']
    LOW = df['low']
    VOL = df['volume']
    AMOUNT = df['amount']

    #
    # {89K趋势策略线, MA1X, V1.55, 2022-12-03}
    #
    # {----------------< 基础数据部分 >----------------}
    # {比对周期长度}
    # C_N:=5;
    C_N = 5
    # {量价关系最低校对比率}
    # C_S:=0.191;
    C_S =0.191
    # {涂画与股价的纵向比率}
    # C_PX:=0.002;
    C_PX =0.002
    # {真阳线}
    # C_ISMALE:=CLOSE > REF(CLOSE,1);
    C_ISMALE = CLOSE > REF(CLOSE,1)
    # {成交量较上一个周期放大}
    # C_VOL:= VOL>REF(VOL,1);
    C_VOL = VOL>REF(VOL,1)
    # {成交量均线周期}
    # VOL_PERIOD:=5;
    VOL_PERIOD = 5
    # {成交量比例}
    # VOLSCALE:=1+C_S;
    VOLSCALE = 1+C_S
    # {高股价或指数的计算方法, 比MAVOL5高出C_S/10且比前一日方法}
    # X_INDEX:=VOL>=MA(VOL, VOL_PERIOD)*(1 + C_S/10);
    X_INDEX = VOL>=(MA(VOL, VOL_PERIOD)*(1 + C_S/10))
    # {一般股价的计算方法}
    # X_GENERAL:=VOL>=MA(VOL, VOL_PERIOD)*VOLSCALE;
    X_GENERAL = VOL>=(MA(VOL, VOL_PERIOD)*VOLSCALE)
    # {指数类或者高股价类的成交量不太可能像个股那样成倍放量, 这里做一个降级处理}
    # X:=IFF(CLOSE>=100, X_INDEX, X_GENERAL) AND C_ISMALE AND C_VOL;
    #X = IFF(CLOSE >= 100, X_INDEX, X_GENERAL) & C_ISMALE & C_VOL
    #X0 = IFF(CLOSE >= 100, X_INDEX, X_GENERAL) & C_ISMALE
    #X = X0 & C_VOL
    x0 = IFF(CLOSE >= 100, X_INDEX, X_GENERAL)
    X = pd.eval('x0 & C_ISMALE & C_VOL')
    # {确定X上一次确立成立是在哪一天}
    # DN: BARSLAST(X),NODRAW;
    DN = BARSLAST(X)
    df['DN'] = DN
    #
    # {强弱提示}
    # XWIDTH:=0.009499432279;
    # PI:=3.141592654;
    # MA3:=MA(CLOSE,5);
    # QR1:=CLOSE-MA3;
    # QR2:=QR1/MA3;
    # QR3:=QR2/XWIDTH;
    # QR4:=ATAN(QR3);
    # QR:QR4*180/PI,NODRAW,COLORLIGRAY;
    # {----------------< A: MA移动平均线 >----------------}
    # P1:=5;
    # P2:=13;
    # P3:=21;
    # P4:=34;
    # P5:=65;
    # P6:=89;
    # P7:=144;
    # MA5:MA(CLOSE,P1),COLORFF8D1E;
    # MA13:MA(CLOSE,P2),COLOR0CAEE6;
    # MA21:MA(CLOSE,P3),COLORE970DC;
    # MA34:MA(CLOSE,P4),COLOR0080FF;
    # MA65:MA(CLOSE,P5),COLOR22C57E;
    # MA89:MA(CLOSE,P6), COLORCCFF00;
    # MA144:MA(CLOSE,P7),COLOR13FFFF;
    #
    # {----------------< C: 止盈止损线 >----------------}
    # {放量上攻作为一个小阶段的起点, 该起点K线的最低价作为止盈止损线}
    # ZS_VOL:=REF(VOL,DN);
    ZS_VOL = REF(VOL, DN)
    # ZS_LOW:=REF(LOW,DN);
    ZS = REF(LOW, DN)
    df['ZS'] = ZS
    # {1. 画止损线, 跌破止损线要清仓, 浅黄色虚线}
    # 止损:DRAWLINE(VOL=ZS_VOL,ZS_LOW,DN>0,ZS_LOW,1), DOTLINE,COLOR13FFFF;
    # {止损: ZS_LOW,DOTLINE, LINETHICK1,COLORCCFF00;}
    # {X0:IFF(止损<0, -1, IFF(止损=0,0,1));}
    # {2. 输出最低价}
    # DRAWNUMBER(VOL=ZS_VOL AND DN=0, LOW*(1-C_PX), ZS_LOW), COLOR13FFFF;
    # {3. 无法显示'突破', 周期已被重置, 需要另想办法}
    # {DRAWTEXT(CLOSE>ZS_VOL AND DN>0, HIGH*(1+C_PX), '突破');}
    #
    # {----------------< D: 压力线 >----------------}
    # {D: 压力线}
    # {0. 压力线, 备用}
    # {H0:=FINDHIGH(HIGH,0,5,1);
    # N31: FINDHIGHBARS(HIGH,0,5,1),NODRAW;
    # V2:=REF(VOL,N31);
    # N3:BARSLAST(VOL=V2 AND N31>0)+1,NODRAW;
    # H1:REF(HIGH,N3),NODRAW;
    # }
    # H0:=HHV(HIGH, C_N);
    H0 = HHV(HIGH, C_N)
    # YL_N:=BARSLAST(C_ISMALE AND HIGH=H0 AND C_VOL);
    Y = pd.eval('C_ISMALE & HIGH==H0 & C_VOL')
    #df['Y'] = Y
    YL_N = BARSLAST(Y)
    # YL_VOL:=REF(VOL,YL_N);
    YL_VOL = REF(VOL, YL_N)
    # YL_HIGH:=REF(HIGH,YL_N);
    YL_HIGH = REF(HIGH, YL_N)
    df['YL'] = YL_HIGH
    # {1. 画压力线, 回踩后拉升收盘有突破的, 第2天买入, 浅蓝色虚线}
    # 压力:DRAWLINE(VOL=YL_VOL,YL_HIGH,YL_N>0,YL_HIGH,1), DOTLINE, COLORCCFF00;
    # {2. 输出最高价}
    # DRAWNUMBER(VOL=YL_VOL AND YL_N=0, HIGH*(1+C_PX), YL_HIGH), COLORCCFF00;
    #
    # {----------------< E: 箱体 >----------------}
    # ZN:=DN,NODRAW;
    # ZS:=ZS_LOW,NODRAW;
    # YN:=YL_N,NODRAW;
    YN = YL_N
    # YH:=YL_HIGH,NODRAW;
    YH = YL_HIGH
    # YV:=YL_VOL,NODRAW;
    YV = YL_VOL
    #
    # {技术性测试支撑}
    # 支撑周期:=IFF(ZN=0,REF(ZS,1)+1,ZN);
    ZN = DN
    支撑周期 = IFF(ZN==0,REF(ZS,1)+1,ZN)
    # 支撑线:=IFF(ZN=0,REF(ZS,1),ZS);
    支撑线 = IFF(ZN==0,REF(ZS,1),ZS)
    # 昨日最高:=REF(HIGH,1);
    昨日最高 = REF(HIGH, 1)
    # 今日最低:=LOW;
    今日最低 = LOW
    # 洗盘力度:=100*(昨日最高-今日最低)/昨日最高;
    洗盘力度 = 100 * (昨日最高 - 今日最低) / 昨日最高
    # 回踩:支撑周期>=2 AND 洗盘力度>=2.33 AND LOW<支撑线 AND CLOSE>=支撑线,COLORYELLOW, NODRAW;
    HC = pandas.eval('支撑周期>=2 & 洗盘力度>=2.33 & LOW<支撑线 & CLOSE>=支撑线')
    df['ma1x.hc'] = HC
    # DRAWICON(回踩>0,LOW,1);
    #
    # {技术性测试抛压}
    # 压力周期:=IFF(YN=0,REF(YN,1)+1,YN),NODRAW;
    压力周期 = IFF(YN==0, REF(YN, 1) + 1, YN)
    # 压力线:=IFF(YN=0,REF(YH,1),YH);
    压力线 = IFF(YN==0, REF(YH, 1), YH)
    # 压力量:=IFF(YN=0,REF(YV,1),YV);
    压力量 = IFF(YN==0, REF(YV, 1), YV)
    # T1:=REF(CLOSE,1)<压力线 OR REF(LOW,1)<压力线 OR LOW<压力线,NODRAW;
    t10 = REF(CLOSE,1)<压力线
    t11 = REF(LOW,1)<压力线
    t12 = LOW<压力线
    T1 = pandas.eval('t10 | t11 | t12')
    # T2:=CLOSE>=压力线,NODRAW;
    T2 = CLOSE >= 压力线
    # {T3:=VOL>测试压力量,NODRAW;}
    # 突破:T1 AND T2 AND 压力周期>=2,COLORLIMAGENTA,NODRAW;
    TP = pandas.eval('T1 & T2 & 压力周期>=2')
    df['ma1x.tp'] = TP
    # DRAWICON(突破>0,LOW,38);
    #
    # {----------------< F: 线性回归 >----------------};
    # HG10:REF(YH,YN+1),NODRAW;
    # HG11:BARSLAST(HIGH=HG10*1),NODRAW;
    # HG20:BARSLAST(HIGH=YH),NODRAW;
    # {REF(YN,1),NODRAW;
    # HG02:YN,NODRAW;
    # HG03:REF(YN,1),NODRAW;
    # HG04:YH-REF(YH,1),NODRAW;
    # HG05:
    # HG1:SLOPE(YH,REF(YH,1));
    # HG2:CLOSE*HG1;}
    #
    # {----------------< Z: 测试代码 >----------------};

    return df



#!/usr/bin/python
# -*- coding: UTF-8 -*-
"""
最基础的工具
"""


# A_YI:=100000000;
# 亿:A_YI;
# 股价:CLOSE<30;
# 市值:FINANCE(40)<(2000*A_YI);
# 涨停比例:IF(FINANCE(3)=2,0.3,IF((FINANCE(3)=4 OR FINANCE(3)=3),0.2,0.1));
def ZTBL(code: str) -> float:
    """
    涨停比例, 只支持A股

    .. note::
      30 68开头的个股涨停比例20%, 其余10%

    :param code:
    :return:
    """
    symbol = code[-6:]
    prefix = symbol[0:2]
    if prefix in ['30', '68']:
        return 0.2
    else:
        return 0.1


def ZTPRICE(close, ztbl) -> float:
    """
    计算涨停价
    :param close:
    :param ztbl:
    :return:
    """
    return close * (1 + ztbl)

# 停牌:DYNAINFO(8)<=0;
# ST股:NAMELIKE('ST') OR NAMELIKE('*ST') OR NAMELIKE('S');
# 真阳线:CLOSE>REF(CLOSE,1);
# 筹码锁定:BARSLASTCOUNT(LFS>REF(LFS,1))>=2 AND LFS>=30;
# 浮筹比例:(WINNER(C*1.1)-WINNER(C*0.9))/WINNER(HHV(H,0))*100;
#
# JXN1:=5;
# JXN2:=10;
# M1:=MA(CLOSE,JXN1);
# M2:=MA(CLOSE,JXN2);
# 均线粘合:ABS(M1/M2-1)<0.01;
#
# {K线数据}
# X_HIGH:HIGH;
# X_LOW:LOW;
# X_OPEN:OPEN;
# X_CLOSE:CLOSE;
# X_VOL:VOL;
# X_AMOUNT:AMOUNT;
#
# {斜率}
# C_WIDTH:0.009499432279;
# C_PI:3.141592654;



#!/usr/bin/python
# -*- coding: UTF-8 -*-
import pandas

from quant1x.base.formula import *


def f89k(data:pandas.DataFrame):
    """
    89K图形指标
    :param code:
    :return:
    """
    df = data
    OPEN = df['open']
    CLOSE = df['close']
    HIGH = df['high']
    LOW = df['low']
    VOL = df['volume']
    AMOUNT = df['amount']

    # {89K图形}
    #
    N1 = 89
    # MA89:=MA(CLOSE,N1);
    MA89 = MA(CLOSE,N1)
    # {计算N1日内最低价}
    # L89:=LLV(LOW,N1);
    L89 = LLV(LOW,N1)
    # {计算N1日内最高价}
    # H89:=HHV(HIGH,N1);
    H89 = HHV(HIGH, N1)
    #
    # {确定⑤}
    # K5:L89,NODRAW,COLORGREEN;
    K5 = L89
    # {确定⑦}
    # N7:BARSLAST(H89=HIGH),NODRAW,COLORLIGRAY;
    N7 = BARSLAST(H89==HIGH)
    # T7:=REF(HIGH,N7);
    T7 = REF(HIGH, N7)
    # K7:T7,NODRAW,COLORRED;
    K7 = T7
    # {确定⑧}
    # T8:=LLV(LOW,N7),NODRAW,COLORGREEN;
    T8 = LLV(LOW,N7)
    # N8:BARSLAST(T8=LOW AND N1>N7),NODRAW,COLORLIGRAY;
    N11 = np.repeat(N1, len(CLOSE))
    N8C1 = T8==LOW
    N8C2 = N11>N7
    N8 = BARSLAST(N8C1 & N8C2)
    # K8:T8,NODRAW,COLORGREEN;
    K8 = T8
    # {确定⑨}
    # T9:=HHV(HIGH,N8);
    T9 = HHV(HIGH, N8)
    # N9X:=BARSLAST(T9=HIGH AND N7>N8);
    N9XC1 = T9 == HIGH
    N9XC2 = N7>N8
    N9X = BARSLAST(N9XC1 & N9XC2)
    # N9:IFF(N9X=0,N9X+1,N9X),NODRAW,COLORLIGRAY;
    N9 = IFF(N9X==0, N9X+1, N9X)
    # K9:REF(HIGH,N9X),NODRAW,COLORRED;
    K9 = REF(HIGH,N9X)
    # {确定⑩}
    # K10:LLV(LOW,N9),NODRAW,COLORGREEN;
    K10 = LLV(LOW,N9)
    # N10:BARSLAST(K10=LOW AND N8>N9),NODRAW,COLORLIGRAY;
    N10C1 = K10==LOW
    N10C2 = N8>N9
    N10 = BARSLAST(N10C1 & N10C2)
    #
    # {比对周期长度}
    # C_N:=5;
    C_N = 5
    # {量价关系最低校对比率}
    # C_S:=0.191;
    C_S = 0.191
    # {涂画与股价的纵向比率}
    # C_PX:=0.002;
    C_PX = 0.002
    # {真阳线}
    # {C_ISMALE:=(CLOSE > REF(CLOSE,1)) AND (CLOSE > OPEN);}
    # C_ISMALE:=CLOSE > REF(CLOSE,1);
    C_ISMALE = CLOSE > REF(CLOSE, 1)
    # {成交量较上一个周期放大}
    # C_VOL:= VOL>REF(VOL,1);
    C_VOL = VOL > REF(VOL, 1)
    # {成交量均线周期}
    # VOL_PERIOD:=5;
    VOL_PERIOD = 5
    # {成交量比例}
    # VOLSCALE:=1+C_S;
    VOLSCALE = 1 + C_S
    # {高股价或指数的计算方法, 比MAVOL5高出C_S/10且比前一日方法}
    # X_INDEX:=VOL>=MA(VOL, VOL_PERIOD)*(1 + C_S/10);
    X_INDEX = VOL >= MA(VOL, VOL_PERIOD) * (1 + C_S / 10)
    # {一般股价的计算方法}
    # X_GENERAL:=VOL>=MA(VOL, VOL_PERIOD)*VOLSCALE;
    X_GENERAL = VOL >= MA(VOL, VOL_PERIOD) * VOLSCALE
    # {指数类或者高股价类的成交量不太可能像个股那样成倍放量, 这里做一个降级处理}
    # X:=IFF(CLOSE>=100, X_INDEX, X_GENERAL) AND C_ISMALE AND C_VOL;
    #X = IFF(CLOSE >= 100, X_INDEX, X_GENERAL) & C_ISMALE & C_VOL
    X0 = IFF(CLOSE>=100, X_INDEX, X_GENERAL)
    X1 = X0 & C_ISMALE
    X = X1 & C_VOL
    # {确定X上一次确立成立是在哪一天}
    # DN:=BARSLAST(X);
    DN = BARSLAST(X)
    # {放量上攻作为一个小阶段的起点, 该起点K线的最低价作为止盈止损线}
    # ZS_VOL:=REF(VOL,DN);
    # ZS_LOW:=REF(LOW,DN);
    ZS_LOW = REF(LOW, DN)
    # ZS:ZS_LOW,NODRAW,COLORLIGRAY;
    # N11:BARSLAST(CLOSE<ZS_LOW AND N9>N10),NODRAW,COLORLIGRAY;
    # {股价突破止损线}
    # {C0:BARSSINCEN(CLOSE<ZS_LOW,DN),NODRAW;}
    # {C0:FINDHIGHBARS(CLOSE,1,DN-1,1),NODRAW;}
    # C00:=BARSLASTCOUNT(CLOSE<ZS_LOW),NODRAW;
    # C01:=BARSLASTCOUNT(CLOSE>=ZS_LOW),NODRAW;
    # C02:=IFF(C01>0,REF(C00,1),C01),NODRAW;
    # C03:=REF(CLOSE,C02-1),NODRAW;
    # C1:BARSLAST(CLOSE>ZS_LOW AND N9>N10),NODRAW,COLORLIGRAY;
    C1C = pandas.eval('CLOSE>ZS_LOW & N9>N10')
    C1 = BARSLAST(C1C)
    # {股价跌破止损线}
    # C2:BARSLAST(CLOSE<ZS_LOW AND N9>N10),NODRAW,COLORLIGRAY;
    C2C = pandas.eval('CLOSE<ZS_LOW & N9>N10')
    C2 = BARSLAST(C2C)
    # C3:N10>0,NODRAW;
    C3 = N10 > 0
    # B0:=C1=0 AND C2=1 AND C3;
    B0 = C1==0
    B1 = C2==1
    B2 = C3
    # B:50*B0,COLORYELLOW;
    B = pandas.eval('B0 & B1 & B2')
    df['f89k.b'] = B
    #
    # {筹码锁定因子}
    # CM0:=50-LFS;
    # CM:CM0,COLORSTICK;
    # CM1:EMA(CM0,3);
    #
    # {斜率}
    # XL:(K7-K9)/(N7-N9),NODRAW;
    # P0:CLOSE,COLORWHITE;
    # P1:-1*XL*N9+K9,COLORCYAN;
    return df




#!/usr/bin/python
# -*- coding: UTF-8 -*-

"""
python机器学习-线性回归(LinearRegression)算法
https://blog.csdn.net/Arwen_H/article/details/82181288
"""

# 导入numpy库
import numpy as np
# 导入pandas库
import pandas as pd
# 导入机器学习库中的线性回归模块
from sklearn.linear_model import LinearRegression

from quant1x.data import D
from quant1x.formula import *


def predict(symbol: str, n: int = 0):
    """
    基础的线性回归
    :param code:
    :return:
    """

    df = D.dataset(symbol)
    df.set_index('date', inplace=True)
    #df['x'] = np.array(df.index)
    #df['x'] = df['open']
    print(df)
    data_length = len(df)
    # 创建一组N行2列的数据, serial_number为数据顺序序列号, close为收盘价
    data = pd.DataFrame({'serial_number': df['open']/REF(df['close'],1),
                         'close': df['close']})
    data.fillna(0.00, inplace=True)
    from matplotlib import pyplot as plt

    # 这里是将数据转化为一个1维矩阵
    data_train = np.array(data['serial_number']).reshape(data['serial_number'].shape[0], 1)
    data_test = data['close']

    # 创建线性回归模型, 拟合面积与价格并通过面积预测价格, 参数默认
    regr = LinearRegression()
    # 拟合数据, serial_number将房屋面积作为x,close价格作为y; 也可以理解用面积去预测价格
    regr.fit(data_train, data_test)
    t = data_length
    #print(df['x'])
    #t = df['x'][-1]
    t = df['open']/REF(df['close'],1)
    t = t[-1]
    a = regr.predict([[t]])
    # 查看预测结果
    print(a)
    # 查看拟合准确率情况,这里的检验是 R^2 , 趋近于1模型拟合越好
    print(regr.score(data_train, data_test))

    # 预测的结果：268.5平的房子价格为8833.54,  R^2 =0.967
    # 我们来画个图看一下数据最后是什么样的

    plt.scatter(data['serial_number'], data['close'])  # 画散点图看实际面积和价格的分布情况
    plt.plot(data['serial_number'],
             regr.predict(np.array(data['serial_number']).reshape(data['serial_number'].shape[0], 1)),
             color='red')  # 画拟合面积与价格的线型图
    plt.show()



# from matplotlib.font_manager import *
#
# _rebuild()

import random

from matplotlib import font_manager  # 使用font-manager来管理字体样式，从而解决中文问题
from matplotlib import pyplot as plt

from quant1x.util import FONT_SimHei

# for font in font_manager.fontManager.ttflist:
#     print(font.name, '-', font.fname)

my_font = font_manager.FontProperties(fname=FONT_SimHei)  # 使用微软雅黑字体

x = range(0, 120)  # 假设时间轴为10点到12点的每分钟为刻度
y = [random.randint(20, 35) for i in range(120)]  # 生成120个数值

plt.figure(figsize=(20, 8), dpi=80)  # 通过初始化figure对象传入figsize指定图像的大小，

plt.plot(x, y)

# 通过xticks方法调整x轴的刻度
_x = list(x)  # 调整时间轴刻度的间隔
# _xtick_labels是自定义的刻度
_xtick_labels = ["10点{}分".format(i) for i in range(60)]  # 10点的范围
_xtick_labels += ["11点{}分".format(i) for i in range(60)]  # 11点的范围

# _x 与 _xtick_labels的数量应该一致，否则数据显示不全
plt.xticks(_x[::5], _xtick_labels[::5], rotation=45,
           fontproperties=my_font)  # rotation 旋转的度数,设置fontproperties属性为找到的字体样式对象

plt.xlabel("时间(s)", fontproperties=my_font)
plt.ylabel("温度(C°)", fontproperties=my_font)
plt.title("xxx市10点~12点每分钟的气温变换", fontproperties=my_font)

plt.show()



#!/usr/bin/python
# -*- coding: UTF-8 -*-
import datetime
import os.path

import akshare as ak
import mootdx.utils.holiday as mooUtils
import pandas
from mootdx.quotes import Quotes
from tqdm import tqdm

from quant1x.data import *


class DataHandler:
    """
    数据
    """

    def __init__(self):
        """
        无参数构造函数
        """
        self.__root = os.path.expanduser(quant1x_home)
        self.__data_cn = os.path.expanduser(quant1x_data_cn)
        self.__data_hk = os.path.expanduser(quant1x_data_hk)
        self.__info_cn = os.path.expanduser(quant1x_info_cn)
        self.__info_hk = os.path.expanduser(quant1x_info_hk)

        # 根路径
        if not os.path.exists(self.__root):
            os.makedirs(self.__root)
        # 数据路径
        self.__path = os.path.expanduser(quant1x_data)
        if not os.path.exists(self.__data_cn):
            os.makedirs(self.__data_cn)
        if not os.path.exists(self.__data_hk):
            os.makedirs(self.__data_hk)
        # 咨询路径
        self.__info = os.path.expanduser(quant1x_info)
        if not os.path.exists(self.__info_cn):
            os.makedirs(self.__info_cn)
        if not os.path.exists(self.__info_hk):
            os.makedirs(self.__info_hk)

        self.__stock_list = {}
        # 自选股csv文件路径
        # 从自选股中获取证券代码列表
        # stock_list.columns = [
        #     "market",
        #     "code",
        #     "name",
        # ]
        zxg_csv = self.__path + '/zxg.csv'
        self.__stock_list = pandas.read_csv(zxg_csv)
        # print(self.__stock_list)

        # 标准市场
        self.__client = Quotes.factory(market='std', multithread=True, heartbeat=True)

    def __NotExistsToMake(self, path):
        """
        如果不存在则创建路径
        :param path:
        :return:
        """
        if not os.path.exists(path):
            os.makedirs(path)

    def apply(self, func_name: str, func) -> pandas.DataFrame:
        """
        自选股迭代
        :param func_name:
        :param func:
        :return:
        """
        total = len(self.__stock_list)
        print("自选股[%s]处理, 共计[%d]:" % (func_name, total))
        values = enumerate(self.__stock_list.values)
        pbar = tqdm(values, total=total)
        df = pandas.DataFrame()
        for key, value in pbar:
            # if key == 2:
            #     break

            code = value[1][2:]
            name = value[2]
            try:
                pbar.set_description_str("[%s]进行中" % code)
                s = func(code, name)
                # df = df.append(s, ignore_index=True)
                df1 = pandas.DataFrame([s])
                df = pandas.concat([df, df1], ignore_index=True)
            except ValueError:
                pass
            finally:
                pbar.set_description_str("[%s]完成" % code)
        pbar.close()
        print("自选股[%s], 处理完成." % func_name)
        return df

    def holiday(self, dt: datetime):
        """
        判断datetime是否假日
        :param dt:
        :return:
        """
        # date = dt.fromtimestamp("%Y-%m-%d")
        date = dt.strftime("%Y-%m-%d")
        return mooUtils.holiday(date)

    def __kline_cn(self, code):
        """
        获取全量历史K线数据
        """
        symbol = code[-6:]
        data = ak.stock_zh_a_hist(symbol=symbol, period="daily", adjust="qfq")
        return data

    def update_history(self):
        """
        更新全量历史数据
        :return:
        """
        total = len(self.__stock_list)
        print("自选股历史K线数据, 共计[%d]:" % total)
        values = enumerate(self.__stock_list.values)
        pbar = tqdm(values, total=total)
        for key, value in pbar:
            # print('key:', key, ', value: ', value)
            code = value[1][2:]
            # print("code:%s" % code)
            pbar.set_description_str("同步[%s]进行中" % code)
            data = self.__kline_cn(code)
            data.to_csv(self.__data_cn + '/' + code + '.csv', index=False)
            pbar.set_description_str("同步[%s]完成" % code)

        pbar.close()
        print("自选股历史K线数据, 处理完成.\n")

    def dataset(self, code) -> pandas.DataFrame:
        """
        读取历史数据
        :param code:
        :return:
        """
        filename = self.__data_cn + '/' + code + '.csv'
        if not os.path.exists(filename):
            data = self.__kline_cn(code)
            data.to_csv(self.__data_cn + '/' + code + '.csv', index=False)
        df = pandas.read_csv(filename)
        # 选择列, 是为了改变表头
        df = df[["日期", "开盘", "收盘", "最高", "最低", "成交量", "成交额"]]
        # 变更表头
        df.columns = ['date', 'open', 'close', 'high', 'low', 'volume', 'amount']
        # 更正排序
        df['date'] = pandas.to_datetime(df['date'])
        #df.set_index('date', inplace=True)

        return df

    def __finance(self, code):
        """
        获取个股基本信息
        :param code:
        :return:
        """
        symbol = code[-6:]
        data = self.__client.finance(symbol=symbol)
        return data

    def update_info(self):
        """
        更新全量个股信息
        :return:
        """

        total = len(self.__stock_list)
        print("自选股基本面信息, 共计[%d]:" % total)
        values = enumerate(self.__stock_list.values)
        pbar = tqdm(values, total=total)
        for key, value in pbar:
            # print('key:', key, ', value: ', value)
            code = value[1][2:]
            # print("code:%s" % code)
            pbar.set_description_str("同步[%s]进行中" % code)
            data = self.__finance(code=code)
            data.to_csv(self.__info_cn + '/' + code + '.csv', index=False)
            pbar.set_description_str("同步[%s]完成" % code)

        pbar.close()
        print("自选股基本面信息, 处理完成.\n")

    def finance(self, code):
        """
        读取本地基本面
        :param code:
        :return:
        """
        filename = self.__info_cn + '/' + code + '.csv'
        if not os.path.exists(filename):
            return
        df = pandas.read_csv(filename)
        return df



#!/usr/bin/env python
"""The setup script."""

try:
    from setuptools import find_packages, setup
except ImportError:
    from distutils.core import find_packages, setup


def parse_requirements(filename):
    line_iter = (line.strip() for line in open(filename))
    return [line for line in line_iter if line and not line.startswith("#")]


with open("README.rst", encoding="utf-8") as readme_file:
    readme = readme_file.read()

with open("HISTORY.rst", encoding="utf-8") as history_file:
    history = history_file.read()

requirements = parse_requirements("requirements.txt")
test_requirements = requirements
# test_requirements.append('pytest')

setup(
    name="quant1x",
    version="0.1.10",
    description="Quant1X量化交易框架",
    long_description=readme,
    author="wangfeng",
    author_email="wangfengxy@sina.cn",
    url="https://gitee.com/quant1x/89k",
    packages=find_packages(include=["quant1x", "quant1x.*"]),
    # include_package_data=True,
    install_requires=requirements,
    license="MIT license",
    zip_safe=False,
    keywords="quant1x",
    entry_points={
        "console_scripts": [
            "quant1x=quant1x.__main__:entry",
        ]
    },
    classifiers=[
        "Development Status :: 2 - Pre-Alpha",
        "Intended Audience :: Developers",
        "License :: OSI Approved :: MIT License",
        "Natural Language :: English",
        "Programming Language :: Python :: 3",
        "Programming Language :: Python :: 3.8",
        "Programming Language :: Python :: 3.9",
        "Programming Language :: Python :: 3.10",
        "Programming Language :: Python :: 3.11",
    ],
    test_suite="tests",
    tests_require=test_requirements,
    setup_requires=requirements,
)



#!/usr/bin/python
# -*- coding: UTF-8 -*-
"""
线性回归
基于线性回归的股票预测(scikit-learn) https://blog.csdn.net/qq_42433311/article/details/121382417
"""

# 线性回归算法一般用于解决"使用已知样本对未知公式参数的估计"类问题
# 获取数据
#     股票数据特征:开盘价(open)、最高价(high)、最低价(low)、收盘价(close)、交易额(volume)
#     及调整后的开盘价(open)、最高价(high)、最低价(low)、收盘价(close)、交易额(volume)
# 数据预处理
#     除权后的数据更能反映数据特征，选择调整后的数据为主要使用的数据特征
#     两个数据特征：HL_PCT(股票最高价与最低价变化百分比)、PCT_change(股票收盘价与最低价的变化百分比)
#     自变量为: close, HL_PCT, PCT_change, volume
#     因变量为: close

import math

import akshare as ak
import numpy as np
import pandas as pandas
from sklearn import model_selection
from sklearn import preprocessing
from sklearn.linear_model import LinearRegression

from quant1x.data import D


def predict(symbol: str, n: int = 0, percent: int = 1):
    """
    预测此后N个交易日的价格趋势

    ----
    :param symbol: 证券代码
    :param n: 预测几天
    :param percent: 预测后起的百分之percent的数据
    :return:
    """

    df = D.dataset(symbol)
    # 选择列, 是为了改变表头
    #df = df[["日期", "开盘", "收盘", "最高", "最低", "成交量", "成交额"]]
    # 变更表头
    #df.columns = ['date', 'open', 'close', 'high', 'low', 'volume', 'amount']
    # 更正排序
    #df['date'] = pandas.to_datetime(df['date'])
    df.set_index('date', inplace=True)

    # 定义预测列变量，存放研究对象的标签名
    forecast_col = 'close'
    # 默认预测1天
    forecast_out = 1
    # 如果预测天数为0, 则使用percent为所有数据量长度的百分比
    if n == 0 and percent > 0:
        forecast_out = int(math.ceil((percent / 100) * len(df)))
    elif n > 0 and percent ==0:
        forecast_out = n
    current_close = df['close'][-1]
    current_date = df.index[-1].strftime("%Y-%m-%d")


    # 只用到df中的下面几个字段
    df = df[['open', 'high', 'low', 'close', 'volume']]
    # 构造两个新列
    df['HL_PCT'] = (df['high'] - df['close']) / df['close'] * 100.0
    df['PCT_change'] = (df['close'] - df['open']) / df['open'] * 100.0
    # 真正用到的特征
    #df = df[['close', 'HL_PCT', 'PCT_change', 'volume']]
    df = df[['close']]
    # 处理空值，这里设置为-99999
    df.fillna(-99999, inplace=True)
    # label代表预测结果，通过让Adj. Close列的数据往前移动1%行来表示
    df['label'] = df[forecast_col].shift(-forecast_out)
    # 生成在模型中使用的数据X,y,以及预测时用到的数据X_lately
    # TODO: 会产生告警: FutureWarning: In a future version of pandas all arguments of DataFrame.drop except for the argument 'labels' will be keyword-only.
    # X = np.array(df.drop(['label'], 1))
    X = np.array(df.drop(labels=['label'], axis=1))
    train_inf = np.isinf(X)
    X[train_inf] = 0.00
    #X = preprocessing.scale(X)
    # 上面生成的label列时留下的最后1%行的数据，这些行并没有label 数据，用作预测时用到的输入数据
    X_lately = X[-(forecast_out+0):]
    X = X[:-forecast_out]
    # 抛弃label列中为空的那些行
    df.dropna(inplace=True)
    # 缺失的值填充
    df.fillna(0.00)
    # 无穷数值处理
    # print(np.isfinite(df).all())
    # False:不包含
    # True:包含
    #print(np.isinf(df).all())

    # while True:
    #     train_inf = np.isinf(df)
    #     df[train_inf] = 0

    y = np.array(df['label'])

    # 先把X，y数据分成两部份，训练和测试
    X_train, X_test, y_train, y_test = model_selection.train_test_split(X, y, test_size=0.2)
    # 生成线性回归对象
    clf = LinearRegression(n_jobs=-1)
    # 开始训练
    clf.fit(X_train, y_train)
    # 用测试数据评估准确性
    accuracy = clf.score(X_test, y_test)
    # 进行预测
    foreca_set = clf.predict(X_lately)
    print(foreca_set, accuracy)
    return current_date, current_close, foreca_set[-1], accuracy
